********************************************************************************
********************************************************************************
*** Dynamic IPW estimates
********************************************************************************
********************************************************************************

************************************************************************************
* Program: Dynamic estimator adjusted to the application 

* Input: 	1. tab_name 
*			2. == 1 impose common support 

* Separate estimation of propensity scores for month 1-10 and joint thereafter
* Aggregate effect over first 12 months
* Covariates used in the application
*************************************************************************************

capture: program drop dyn_application
program define dyn_application

local common  = `2'
local periods = 24
local sep     = 10
local trim    = 0.01

set more off
local est_controls "male medel gam married gymn egymn child7_15 child4_6 child0_3 ui_elig ilok days_m1 days_m2 loneink_m1 loneink_m2 loneink_m3 sth gbg skane north south y1996 y1997 y1998"
				
	replace Ts_m = 1000 if Ts_m > `periods' | Ts_m == .	
	replace Tu_m = `periods'   if Tu_m > `periods' 
			
	gen nr = _n
	preserve
	
	// Estimation of propensity scores 
	
	* Separate estimation for periods 1-10
	forvalues t = 1/`sep' {
		gen D   = Ts_m == `t'
		gen inn = Tu_m >= `t' & Ts_m >= `t'
		logit D `est_controls' if inn == 1, iterate(200)
		predict _PS`t'
		drop D inn
	}
	keep nr _PS*
	save _apa, replace
	
	restore 
	
	* Joint estimation for later periods
	expand `periods'
	bysort nr: gen mm = _n

	gen inn = 1
	replace inn = 0 if mm > Tu_m 
	replace inn = 0 if mm > Ts_m
	gen D = Ts_m == mm
	
	logit D i.mm `est_controls' if inn == 1 & mm > `sep', iterate(200)		
	predict _PS	
	drop D inn
	reshape wide _PS, i(nr) j(mm)
	
	forvalues t = 1/`sep' {
		capture: drop _PS`t'
	}
	
	merge 1:1 nr using _apa, nogenerate
	drop nr
	save _apa, replace

	// Estimates for month to treatment and outcomes t to t+10

	forvalues t = 1/`periods' {
		forvalues u = 1/10 {
			  use _apa, clear
              keep if Ts_m == `t' | (Tu_m >= `t' & Ts_m == 1000)              // Select duration
							 
			  * Adjust the wage income by time since the start of the program
			    gen yin = year(start)
		        gen min = month(start)

				gen yp  = yin
				gen mm = min+`t'-1
				replace yp = yp+1  	if mm > 12 
				replace mm = mm-12  if mm > 12 
				replace yp = yp+1  	if mm > 12 
				replace mm = mm-12  if mm > 12
				replace yp = yp+1  	if mm > 12 
				replace mm = mm-12  if mm > 12
				gen n = yp-yin+1	 // # years to shift yearly incomes				

				sum n
				local a = r(max)
				gen 	Y = loneink_`u'
					forvalues n = 1/`a' {
							local nn = `n'+`u'-1
							replace Y = loneink_`nn' if n == `n'
					}
				drop yin min yp n mm	
											  
				// ATET estimates
                keep Y Ts_m Tu_m _PS*
                gen D = Ts_m == `t'

				// Weights
				sum D
				gen w1  = 1
				
				gen w0  = _PS`t'/(1-_PS`t')
				local tt = `t'+1
				forvalues k = `tt'/`periods' {
					replace w0 = w0/(1-_PS`k') if Tu_m >= `k'
				}
				drop _PS*
								
				// Common support (Huber et al. % rule), only for non-treated - ATET
				if `common' == 1 {
					*sum w1 if D == 1
					*drop if w1 > r(sum)*`trim' 	
					
					sum w0 if D == 0
					drop if w0 > r(sum)*`trim' 	
				}
			
				// ATET Estimate
				sum w1 if D == 1
				local ww1 = r(N)
				sum Y  if D == 1
				gen u1 = r(sum)/`ww1'
				
				sum w0 if D == 0
				local ww0 = r(sum)
				gen w0y = w0*Y
				sum w0y if D == 0
				
				gen u0 = r(sum)/`ww0'
				gen est = u1-u0
							  
				  
				// Frequency and variables
				sum est if D == 1
				gen freq = r(N)
				keep freq est u1 u0 Y
				keep if _n == 1
				gen dur = `t'
				gen u   = `u'
                               
				replace u1  = . if Y == .
				replace u0  = . if Y == .
				replace est = . if Y == .
				drop Y

				if `t' != 1 | `u' != 1 append using "`1'"
				save "`1'" , replace
         }
	} 

* Aggregated effects - over first 12 months

use "`1'", clear
		drop if dur > 12	
        egen tot = total(freq), by(u)
        gen w = freq/tot
        gen w_est = w*est
        collapse (sum) w_est w, by(u)
        gen est = w_est/w
save "`1'_wagg", replace

end


**************** Dynamic IPW - Estimates ****************

use "_lr_anlysis_ecma" if uniform()< 1 | Ts_m !=. , clear
qui dyn_application main_results 0
list


**************** Dynamic IPW - Bootstrap inference *******

// Extract bootstrap replicates
 
set more off
set seed 20160607

forvalues rep = 1/500 {
	di "Replication " `rep'
	use "_lr_anlysis_ecma" if uniform()<1 | Ts_m !=. , clear
	bsample _N
	save "_samp_`rep'_ecma", replace
}

// Estimates

forvalues rep = 1/500 {
	di "Replication " `rep'
	use "_samp_`rep'_ecma" if uniform()<0.001 | Ts_m !=. , clear
	qui dyn_application _res_`rep' 0
}
