clear all 

******************************************************************
*** MONTE CARLO PROGRAMS *********************************
******************************************************************




*******************************************************************
* General Monte Carlo program: estimator name as input + dgp parameters
*******************************************************************

capture program drop mc_alt
program define mc_alt
	args app N_sim N_reps N_obs hazT0 hazS0 betaT betaS bu bx bw N_periods
	
qui {	
	set more off
	tempfile _n1
	
	forvalues nn = 1/`N_sim' {		
		dat `N_obs' `hazT0' `hazS0' `betaT' `betaS' `bu' `bx' `bw' `N_periods'
		`app' X
		gen rep = `nn'
		if `nn' != 1 append using `_n1'
		save `_n1', replace
		noi display "`nn'"
		noi display "$S_TIME  $S_DATE"
	}
	
	egen nr_sim = count(rep)
	egen meanest = mean(est)
	gen se   = (est-meanest)^2			
	
	gen true = `bw'
	gen bias = (est-true)
	
	*gen size = abs(est/est_se) >1.96 
	*order est est_se size bias se nr_sim 
}

end	





*******************************************************************
* MC Alternative estimators: Send estimator name as input
*******************************************************************

capture program drop mc_alt_boot
program define mc_alt_boot
	args app N_sim N_reps N_obs hazT0 hazS0 betaT betaS bu bx bw N_periods
	
qui {	
	set more off
	tempfile _n1 dat est boot
		
	forvalues nn = 1/`N_sim' {
		
		* Estim
		dat `N_obs' `hazT0' `hazS0' `betaT' `betaS' `bu' `bx' `bw' `N_periods'
		save `dat', replace
		`app' X
		gen a = 1
		save `est', replace
		
		* Bootstrap
		if `N_reps' > 0 {
			forvalues rep = 1/`N_reps' {
					use `dat', clear					
					bsample _N
					`app' X
					keep est
					rename est est_boot
					gen sim = `rep'
					if `rep' != 1 append using `boot'
					save `boot', replace
			}
		}
		gen a = 1
		joinby a using `est'
		drop a
			
		gen rep = `nn'
		if `nn' != 1 append using `_n1'
		save `_n1', replace
		noi display "`nn'"
		noi display "$S_TIME  $S_DATE"
	}
	
}

end	





************************************************************
**************** Generate data ****************************
************************************************************

* 1:   	Number of observations
* 2-5: 	DGP input, treatment and outcome duration
* 6-9:  DGP input, long-run outcome
* 10:	Number of periods for the treatment

* General DGP:	hazTu  = f(hazT0 + betaT*X + Vu)			[in the paper: f(-2.5+X+v_u)]
*				hazTS  = f(hazS0 + betaS*X)					[in the paper: f(alpha_s+beta_s*X)]
*				f	   = [1+exp(-h)]^-1
*				Y	   = 100 + bx*X + bw*I(Tu>=Ts)+ bu*Vu + Vy 	
*	    [in the paper: 100 + beta_Y*X + delta*I(Tu>=Ts)+ beta_u*v_u + v_y

program define dat
	args N_obs hazT0 hazS0 betaT betaS bu bx bw N_periods

	createb_data `N_obs' `hazT0' `hazS0' `betaT' `betaS' 
	rename censor C
	replace S = . if S > `N_periods'
	gen W = S<=T
	replace Y = 100 +`bx'*(X)+`bw'*W+`bu'*(Y)+sqrt(5)*rnormal()			
end 

**********************************************************************
* Generate treatment and outcome duration data 
	* Input:  `N_obs' `hazT0' `hazS0' `betaT' `betaS' 
	* Output:  Dataset _data with T: Exit time S: Treatment time X: Covariate

program define createb_data

clear
set obs 1
gen a  = `1'
gen T  = 0
gen S  = 0
gen X  = 0
gen Y  = 0
expand(a)
drop a

mata: create2(`1', `2', `3', `4', `5') 
gen censor = 0

end

* Mata function used to generate outcome and treatment duration

mata
	function create2(N_obs, hazT0, hazS0, betaT, betaS)
	{
		n     = N_obs
		i     = 1
		while (i<=n) { 
			X     = 2*uniform(1,1)-1
		    VS    = 2*uniform(1,1)-1
			VT    = 2*uniform(1,1)-1
			haz0  = (1+exp(-(hazT0+betaT*X+VT)))^-1
			haz1  = (1+exp(-(hazT0+betaT*X+VT)))^-1
			hazS  = (1+exp(-(hazS0+betaS*X+VS)))^-1
			
			// Treatment duration
			trans = 0
			S     = 0
			while (trans == 0) {      
				x = uniform(1,1)
				if (x<hazS) {
					trans = 1
				}
				S = S+1
			}	
			
			// Outcome duration
			trans  = 0
			T     = 0
			while (trans == 0) {     
				b = uniform(1,1)
				if (T>=S-1) {
					haz = haz1
				}
				else {
					haz = haz0
				}
				if (b<haz) {
					trans = 1
				}
				T = T+1
			}	
			Y = VT
			_st_store(i,1,T)
			_st_store(i,2,S)
			_st_store(i,3,X)
			_st_store(i,4,Y)
			i = i+1
		} 
	}
end



*********************************************************
************** ESTIMATORS ********************
**********************************************************

************************************************************************************************
* Program: DYNIPW with no-joint estimation of propensity score

* Input: 	1. Covariates

* Data:		S - treatment duration
*			T - outcome duration
*			Y - long-run outcome

* Set directly in the code (here set for bias analyses)
* 1. Number of follow up periods, here 12
* 2. Number of period for separate PS estimation, here separate each period
* 3. Common support percent cut-off, here 1%
************************************************************************************************

capture: program drop dyn_ipw_nojoint_mc_common
program define dyn_ipw_nojoint_mc_common

local covs `*'

local common  = 1
local periods = 12		
local sep     = 12
local trim    = 0.01

rename S Ts_m
rename T Tu_m

set more off

	replace Ts_m = 1000 if Ts_m > `periods' | Ts_m == .	
	replace Tu_m = `periods'   if Tu_m > `periods' 
			
	gen nr = _n

* Estimate propensity score for each period
	forvalues t = 1/`sep' {
		gen D   = Ts_m == `t'
		gen inn = Tu_m >= `t' & Ts_m >= `t'
		logit D `covs' if inn == 1
		predict _PS`t'
		drop D inn
	}
	
	tempfile _apa
	save `_apa', replace
	
tempfile _1		

* Construct estimates for each duration (outcome defined by Y variable)
 set more off
 forvalues t = 1/`periods' {
	use `_apa', clear
                             
    // Select the sample
    keep if Ts_m == `t' | (Tu_m < Ts_m )            
	drop if Tu_m < `t'
	drop if Ts_m < `t'								  
							  
	keep Y Ts_m Tu_m _PS*
    gen D = Ts_m == `t'

    // Weights
    sum D
    gen w1  = 1
	
    gen w0  = _PS`t'/(1-_PS`t')
    local tt = `t'+1
    forvalues k = `tt'/`periods' {
		replace w0 = w0/(1-_PS`k') if Tu_m >= `k'
    }
    drop _PS*
					
	// Common support (Huber et al. % rule), only for non-treated - ATET
	*sum w1 if D == 1
	*drop if w1 > r(sum)*`trim' 	
	
	sum w0 if D == 0
	drop if w0 > r(sum)*`trim' 	
	
	// ATET Estimate
	sum w1 if D == 1
	local ww1 = r(N)
    sum Y  if D == 1
	gen u1 = r(sum)/`ww1'
	
	sum w0 if D == 0
    local ww0 = r(sum)
	gen w0y = w0*Y
	sum w0y if D == 0
	
	gen u0 = r(sum)/`ww0'
	gen est = u1-u0
    
	// Frequency and variables
	sum est if D == 1
    gen freq = r(N)
	keep freq est u1 u0 Y
    keep if _n == 1
	gen dur = `t'
	drop Y

    if `t' != 1 append using `_1'
    save `_1', replace
 }
 

* Calculate aggregated effects

use `_1', clear
		 list
         egen tot = total(freq)

         gen w = freq/tot
         gen w_est = w*est
		 gen w_u1  = w*u1
		 gen w_u0  = w*u0
		 
         collapse (sum)  w_est w w_u1 w_u0
         gen est = w_est/w
end



************************************************************************************************
* Program: DYNIPW with no-joint estimation of propensity score. 

* Input: 	1. Covariates

* Data:		S - treatment duration
*			T - outcome duration
*			Y - long-run outcome

* Set directly in the code (here set for bootstrap analyses):
* 1. Number of follow up periods, here 12
* 2. Number of period for separate PS estimation, here separate each period
* 3. Common support percent cut-off, here 1%
************************************************************************************************

capture: program drop dyn_ipw_nojoint_mc_common_boot
program define dyn_ipw_nojoint_mc_common_boot

local covs `*'

local common  = 0
local periods = 4		
local sep     = 4
local trim    = 0.05

rename S Ts_m
rename T Tu_m

set more off

	replace Ts_m = 1000 if Ts_m > `periods' | Ts_m == .	
	replace Tu_m = `periods'   if Tu_m > `periods' 
			
	gen nr = _n

* Estimate propensity score for each period
	forvalues t = 1/`sep' {
		gen D   = Ts_m == `t'
		gen inn = Tu_m >= `t' & Ts_m >= `t'
		logit D `covs' if inn == 1
		predict _PS`t'
		drop D inn
	}
	
	tempfile _apa
	save `_apa', replace
	
tempfile _1		

* Construct estimates for each duration (outcome defined by Y variable)
 set more off
 forvalues t = 1/`periods' {
	use `_apa', clear
                             
    // Select the sample
    keep if Ts_m == `t' | (Tu_m < Ts_m )            
	drop if Tu_m < `t'
	drop if Ts_m < `t'								  
							  
	keep Y Ts_m Tu_m _PS*
    gen D = Ts_m == `t'

    // Weights
    sum D
    gen w1  = 1
	
    gen w0  = _PS`t'/(1-_PS`t')
    local tt = `t'+1
    forvalues k = `tt'/`periods' {
		replace w0 = w0/(1-_PS`k') if Tu_m >= `k'
    }
    drop _PS*
					
	// Common support (Huber et al. % rule), only for non-treated - ATET
	if `common' == 1 {
		*sum w1 if D == 1
		*drop if w1 > r(sum)*`trim' 	
		
		sum w0 if D == 0
		drop if w0 > r(sum)*`trim' 	
	}
		
	// ATET Estimate
	sum w1 if D == 1
	local ww1 = r(N)
    sum Y  if D == 1
	gen u1 = r(sum)/`ww1'
	
	sum w0 if D == 0
    local ww0 = r(sum)
	gen w0y = w0*Y
	sum w0y if D == 0
	
	gen u0 = r(sum)/`ww0'
	gen est = u1-u0
    
	// Frequency and variables
	sum est if D == 1
    gen freq = r(N)
	keep freq est u1 u0 Y
    keep if _n == 1
	gen dur = `t'
	drop Y

    if `t' != 1 append using `_1'
    save `_1', replace
 }
 

* Calculate aggregated effects

use `_1', clear
		 list
         egen tot = total(freq)

         gen w = freq/tot
         gen w_est = w*est
		 gen w_u1  = w*u1
		 gen w_u0  = w*u0
		 
         collapse (sum)  w_est w w_u1 w_u0
         gen est = w_est/w
end





**********************************************************
************** Static IPW Estimator **********************
**********************************************************

* IPW directly on observed treatment status [W if treated before exit]

* Common support program 
capture program drop common
program define common
	sum _PS if W == 1
	local min = r(min)
	local max = r(max)
	sum _PS if W == 0
	local min = max(`min',r(min))
	local max = min(`max',r(max))
	drop if _PS < `min' | _PS > `max'
end

* Etimation program
program define naive_ipw
local covs `*'
	qui {

	// Estimate propensity score
	logit W `covs'
	predict _PS
	sum _PS

	// Common-support
	*common
	  
	// Weigths and estimates
	gen w1 = (1/1)
	sum w1 if W == 1
	replace w1 = w1*Y/r(sum)
	sum w1 if W == 1
	gen s1 = r(sum)

	gen      w0 = _PS/(1-_PS)
	sum w0 if W == 0
	replace w0 = w0*Y/r(sum)
	sum w0 if W == 0
	gen s0 = r(sum)
	gen est = s1-s0
	drop if _n>1
	keep est s1 s0

	}
end
